﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using MySql.Data.MySqlClient;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using YouthBooking.Models;

// For more information on enabling Web API for empty projects, visit https://go.microsoft.com/fwlink/?LinkID=397860

namespace YouthBooking.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class BookingController : ControllerBase
    {
        private readonly IConfiguration Configuration;
        private string _connectionString;

        public BookingController(IConfiguration configuration)
        {
            Configuration = configuration;
            var server = Configuration["MySQLConnection:Server"];
            var userid = Configuration["MySQLConnection:UserId"];
            var password = Configuration["MySQLConnection:Password"];
            var databasename = Configuration["MySQLConnection:Database"];
            _connectionString = $"server={server};userid={userid};password={password};database={databasename}";
        }

        // GET api/<CustomerController>/retrieve
        [Route("retrieve")]
        [HttpGet()]
        public ActionResult<object> RetrieveBookings(string username, DateTime fromDate, DateTime? toDate = null)
        {
            if (!toDate.HasValue)
            {
                toDate = fromDate.AddDays(1);
            }

            fromDate = DateTime.SpecifyKind(fromDate, DateTimeKind.Local);
            toDate = DateTime.SpecifyKind(toDate.Value, DateTimeKind.Local);

            var con = new MySqlConnection(_connectionString);
            con.Open();

            StringBuilder sqlBuild = new StringBuilder();
            sqlBuild.AppendLine($"SELECT bk.id, bk.startdatetime, bk.enddatetime, bk.roomid, bk.username, bk.info, bk.statusflg, rm.roomname FROM booking bk, room rm ");
            sqlBuild.AppendLine($"where rm.roomid = bk.roomid ");

            if (username == "*")
            {
                sqlBuild.AppendLine($"AND (startdatetime BETWEEN '{fromDate.ToString("yyyy-MM-dd HH:mm:ss")}' AND '{toDate.Value.ToString("yyyy-MM-dd HH:mm:ss")}')");
            }
            else
            {
                sqlBuild.AppendLine($"AND username='{username}' AND (startdatetime BETWEEN '{fromDate.ToString("yyyy-MM-dd HH:mm:ss")}' AND '{toDate.Value.ToString("yyyy-MM-dd HH:mm:ss")}')");
            }

            string sql = sqlBuild.ToString();
            var cmd = new MySqlCommand(sql, con);

            MySqlDataReader rdr = cmd.ExecuteReader();
            List<BookingDto> bookings = new List<BookingDto>();

            while (rdr.Read())
            {
                BookingDto booking = new() { Id = rdr.GetUInt32(0), StartDateTime = rdr.GetDateTime(1).ToLocalTime(), EndDateTime = rdr.GetDateTime(2).ToLocalTime(), RoomId = rdr.GetUInt32(3), Username = rdr.GetString(4), Info = rdr.GetString(5), StatusFlg = rdr.GetUInt32(6), RoomName = rdr.GetString(7) };
                bookings.Add(booking);
            }

            return Ok(bookings);
        }

        // GET api/<CustomerController>/retrieve
        [Route("getbooking")]
        [HttpGet()]
        public ActionResult<object> GetBooking(string username, int bookingId)
        {
            var con = new MySqlConnection(_connectionString);
            con.Open();

            StringBuilder sqlBuild = new StringBuilder();
            sqlBuild.AppendLine($"SELECT bk.id, bk.startdatetime, bk.enddatetime, bk.roomid, bk.username, bk.info, bk.statusflg, rm.roomname FROM booking bk, room rm ");
            sqlBuild.AppendLine($"where rm.roomid = bk.roomid ");
            sqlBuild.AppendLine($"AND username='{username}' AND id = '{bookingId}'");

            string sql = sqlBuild.ToString();
            var cmd = new MySqlCommand(sql, con);

            MySqlDataReader rdr = cmd.ExecuteReader();
            if (rdr.Read())
            {
                BookingDto booking = new() { Id = rdr.GetUInt32(0), StartDateTime = rdr.GetDateTime(1).ToLocalTime(), EndDateTime = rdr.GetDateTime(2).ToLocalTime(), RoomId = rdr.GetUInt32(3), Username = rdr.GetString(4), Info = rdr.GetString(5), StatusFlg = rdr.GetUInt32(6), RoomName = rdr.GetString(7) };                
                return Ok(booking);
            }
            else
            {
                return BadRequest(new { ErrorCode = 404, ErrorMessage = "Booking not found" });
            }            
        }

        // POST api/<CustomerController>
        [Route("new")]
        [HttpPost]
        public ActionResult<object> NewBooking([FromBody] BookingDto booking)
        {
            try
            {
                var con = new MySqlConnection(_connectionString);
                con.Open();

                string sql = $"INSERT INTO booking (startdatetime, enddatetime, roomid, username, info) VALUES ('{booking.StartDateTime.ToString("yyyy-MM-dd HH:mm:ss")}', '{booking.EndDateTime.ToString("yyyy-MM-dd HH:mm:ss")}', '{booking.RoomId}', '{booking.Username}', '{booking.Info}');";
                var cmd = new MySqlCommand(sql, con);

                if (cmd.ExecuteNonQuery() == 1)
                {
                    return Ok(new { ErrorCode = 0, ErrorMessage = "OK", Data = booking });
                }
                else
                {
                    return Ok(new { ErrorCode = 400, ErrorMessage = "Booking not inserted" });
                }
            }
            catch(Exception e)
            {
                string msg = e.InnerException != null ? e.InnerException.Message : e.Message;
                return Ok(new { ErrorCode = -1, ErrorMessage = "Booking not inserted - " + msg });
            }            
        }

        // PUT api/<CustomerController>
        [Route("edit")]
        [HttpPut]
        public ActionResult<object> EditBooking([FromBody] BookingDto booking)
        {
            return BadRequest();
        }

        // DELETE api/<CustomerController>/<username>
        [Route("delete")]
        [HttpDelete()]
        public ActionResult<object> DeleteBooking(string username, uint bookingId)
        {
            var con = new MySqlConnection(_connectionString);
            con.Open();

            string sql = $"DELETE FROM booking WHERE username='{username}' AND id={bookingId};";
            var cmd = new MySqlCommand(sql, con);

            if (cmd.ExecuteNonQuery() == 1)
            {
                return Ok(new { ErrorCode = 0, ErrorMessage = "OK" });
            }
            else
            {
                return BadRequest(new { ErrorCode = 400, ErrorMessage = "Booking not deleted" });
            }
        }
    }
}
