﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using MySql.Data.MySqlClient;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using YouthBooking.Models;

// For more information on enabling Web API for empty projects, visit https://go.microsoft.com/fwlink/?LinkID=397860

namespace YouthBooking.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class RoomController : ControllerBase
    {
        private readonly IConfiguration Configuration;
        private string _connectionString;

        public RoomController(IConfiguration configuration)
        {
            Configuration = configuration;
            var server = Configuration["MySQLConnection:Server"];
            var userid = Configuration["MySQLConnection:UserId"];
            var password = Configuration["MySQLConnection:Password"];
            var databasename = Configuration["MySQLConnection:Database"];
            _connectionString = $"server={server};userid={userid};password={password};database={databasename}";
        }

        // GET api/<RoomController>/isroomfree
        [Route("isroomfree")]
        [HttpGet()]
        public ActionResult<object> IsRoomFree(int roomId, DateTime fromDate, DateTime toDate)
        {
            fromDate = fromDate.ToLocalTime();
            toDate = toDate.ToLocalTime();
            var con = new MySqlConnection(_connectionString);
            con.Open();

            StringBuilder sqlBuild = new StringBuilder();
            sqlBuild.AppendLine($"SELECT bk.id, bk.startdatetime, bk.enddatetime, bk.roomid, bk.username, bk.info, bk.statusflg, rm.roomname FROM booking bk, room rm ");
            sqlBuild.AppendLine($"where rm.roomid = bk.roomid ");
            sqlBuild.AppendLine($"AND rm.roomid={roomId} ");
            sqlBuild.AppendLine($"AND ( ");
            sqlBuild.AppendLine($"      (startdatetime BETWEEN '{fromDate.ToString("yyyy-MM-dd HH:mm:ss")}' AND '{toDate.ToString("yyyy-MM-dd HH:mm:ss")}')");
            sqlBuild.AppendLine($"      OR (enddatetime BETWEEN '{fromDate.ToString("yyyy-MM-dd HH:mm:ss")}' AND '{toDate.ToString("yyyy-MM-dd HH:mm:ss")}')");
            sqlBuild.AppendLine($"    )");


            string sql = sqlBuild.ToString();
            var cmd = new MySqlCommand(sql, con);

            MySqlDataReader rdr = cmd.ExecuteReader();
            List<BookingDto> bookings = new List<BookingDto>();

            while (rdr.Read())
            {
                BookingDto booking = new() { Id = rdr.GetUInt32(0), StartDateTime = rdr.GetDateTime(1).ToLocalTime(), EndDateTime = rdr.GetDateTime(2).ToLocalTime(), RoomId = rdr.GetUInt32(3), Username = rdr.GetString(4), Info = rdr.GetString(5), StatusFlg = rdr.GetUInt32(6), RoomName = rdr.GetString(7) };
                bookings.Add(booking);
            }
            
            return Ok(new { isFree = (bookings.Count == 0), data = bookings });
        }

        // GET api/<RoomController>/freerooms
        [Route("freerooms")]
        [HttpGet()]
        public ActionResult<object> FreeRooms(DateTime fromDate, DateTime toDate)
        {
            fromDate = fromDate.ToLocalTime();
            toDate = toDate.ToLocalTime();
            var con = new MySqlConnection(_connectionString);
            con.Open();

            StringBuilder sqlBuild = new StringBuilder();

            sqlBuild.AppendLine($"SELECT roomid, roomname, info FROM room WHERE NOT (roomid IN (");
            sqlBuild.AppendLine($"SELECT bk.roomid FROM booking bk, room rm ");
            sqlBuild.AppendLine($"where rm.roomid = bk.roomid ");            
            sqlBuild.AppendLine($"AND ( ");
            sqlBuild.AppendLine($"      (startdatetime BETWEEN '{fromDate.ToString("yyyy-MM-dd HH:mm:ss")}' AND '{toDate.ToString("yyyy-MM-dd HH:mm:ss")}')");
            sqlBuild.AppendLine($"      OR (enddatetime BETWEEN '{fromDate.ToString("yyyy-MM-dd HH:mm:ss")}' AND '{toDate.ToString("yyyy-MM-dd HH:mm:ss")}')");
            sqlBuild.AppendLine($"    )))");


            string sql = sqlBuild.ToString();
            var cmd = new MySqlCommand(sql, con);

            MySqlDataReader rdr = cmd.ExecuteReader();
            List<RoomDto> rooms = new List<RoomDto>();

            while (rdr.Read())
            {
                RoomDto room = new() { RoomId = rdr.GetUInt32(0), Roomname = rdr.GetString(1), Info = rdr.GetString(2) };
                rooms.Add(room);
            }

            return Ok(rooms);            
        }

        [Route("rooms")]
        [HttpGet()]
        public ActionResult<object> GetRooms()
        {
            var con = new MySqlConnection(_connectionString);
            con.Open();

            StringBuilder sqlBuild = new StringBuilder();
            sqlBuild.AppendLine($"SELECT roomid, roomname, info FROM room ");

            string sql = sqlBuild.ToString();
            var cmd = new MySqlCommand(sql, con);

            MySqlDataReader rdr = cmd.ExecuteReader();
            List<RoomDto> rooms = new List<RoomDto>();
            while (rdr.Read())
            {
                RoomDto room = new() { RoomId = rdr.GetUInt32(0), Roomname = rdr.GetString(1), Info = rdr.GetString(2) };
                rooms.Add(room);
            }

            return Ok(rooms);
        }
        
        [Route("new")]
        [HttpPost]
        public ActionResult<object> NewRoom([FromBody] RoomDto room)
        {
            try
            {
                var con = new MySqlConnection(_connectionString);
                con.Open();

                string sql = $"INSERT INTO room (roomid, roomname, info, userrights) VALUES ({room.RoomId}, '{room.Roomname}', '{room.Info}', 1);";
                var cmd = new MySqlCommand(sql, con);

                if (cmd.ExecuteNonQuery() == 1)
                {
                    room.UserRights = 1;
                    return Ok(new { ErrorCode = 0, ErrorMessage = "OK", Data = room });
                }
                else
                {
                    return BadRequest(new { ErrorCode = 400, ErrorMessage = "Room not inserted" });
                }
            }
            catch (Exception e)
            {
                string msg = e.InnerException != null ? e.InnerException.Message : e.Message;
                return BadRequest(new { ErrorCode = -1, ErrorMessage = "Room not inserted - " + msg });
            }
        }
    }
}
