﻿-- create the Youth Booking database schema
CREATE SCHEMA youthbooking DEFAULT CHARACTER SET utf8 ;

-- create the Customer table
CREATE TABLE youthbooking.customer (
  id INT UNSIGNED NOT NULL AUTO_INCREMENT,
  username VARCHAR(45) CHARACTER SET 'utf8' NOT NULL COMMENT 'User name',
  firstname VARCHAR(45) CHARACTER SET 'utf8' NULL COMMENT 'First name',
  lastname VARCHAR(45) CHARACTER SET 'utf8' NULL COMMENT 'Last name',
  email VARCHAR(80) CHARACTER SET 'utf8' NULL COMMENT 'Email',
  userrights INT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'User rights flags for rooms 1=normal',
  adminrights INT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'Administrator rights flags 1=customer',
  password VARCHAR(255) CHARACTER SET 'utf8',
  UNIQUE INDEX id_UNIQUE (id ASC) VISIBLE,
  PRIMARY KEY (username),
  UNIQUE INDEX username_UNIQUE (username ASC) VISIBLE)
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8
COMMENT = 'Customer table for booking account.';

INSERT INTO youthbooking.customer (username, firstname, lastname, email, userrights, adminrights) VALUES ('admin', ' ', ' ', ' ', 4294967295, 4294967295);
INSERT INTO youthbooking.customer (username, firstname, lastname, email, userrights, adminrights) VALUES ('test1', 'First1 ', 'Last1 ', 'test1@test.com', 1, 1);
INSERT INTO youthbooking.customer (username, firstname, lastname, email, userrights, adminrights) VALUES ('test2', 'First2 ', 'Last2 ', 'test2@test.com', 1, 1);

CREATE TABLE youthbooking.booking (
  id INT UNSIGNED NOT NULL AUTO_INCREMENT COMMENT 'Booking identifier',
  startdatetime DATETIME NOT NULL COMMENT 'Start date and time of booking',
  enddatetime DATETIME NOT NULL COMMENT 'End date and time of booking',
  roomid INT UNSIGNED NOT NULL COMMENT 'Room identifier for booking',
  username VARCHAR(45) CHARACTER SET 'utf8' NOT NULL COMMENT 'Username for booking',
  info VARCHAR(255) CHARACTER SET 'utf8' NULL COMMENT 'General information',
  statusflg INT NOT NULL DEFAULT 0 COMMENT 'Status 0=inactive, 1=active',
  UNIQUE INDEX id_UNIQUE (id ASC) VISIBLE,
  PRIMARY KEY (startdatetime, enddatetime, roomid))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8
COMMENT = 'Table containing the bookings';

CREATE TABLE youthbooking.room (
  roomid INT UNSIGNED NOT NULL AUTO_INCREMENT COMMENT 'Room identifier',
  roomname VARCHAR(45) CHARACTER SET 'utf8' NOT NULL COMMENT 'Room name identifier',
  info VARCHAR(255) CHARACTER SET 'utf8' NULL COMMENT 'Information and description',
  userrights INT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'Room rights for users',
  UNIQUE INDEX roomid_UNIQUE (roomid ASC) VISIBLE,
  PRIMARY KEY (roomname))
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8
COMMENT = 'Rooms table. List of rooms to book.';

ALTER TABLE youthbooking.booking
ADD CONSTRAINT fk_roomid
FOREIGN KEY (roomid) REFERENCES room(roomid);

INSERT INTO youthbooking.room (roomid, roomname, info, userrights) VALUES ('1', 'cardiff.main', 'Cardiff Main Hall', '1');
INSERT INTO youthbooking.room (roomid, roomname, info, userrights) VALUES ('2', 'cardiff.second', 'Cardiff Second Hall', '1');
INSERT INTO youthbooking.room (roomid, roomname, info, userrights) VALUES ('3', 'cardiff.third', 'Cardiff Third Hall', '1');


INSERT INTO youthbooking.booking (id, startdatetime, enddatetime, roomid, username, info, statusflg) VALUES ('1', '2022-05-01 10:00', '2022-05-01 10:30', '1', 'test1', 'First test booking', '1');
INSERT INTO youthbooking.booking (startdatetime, enddatetime, roomid, username, info, statusflg) VALUES ('2022-05-01 10:30', '2022-05-01 11:00', '1', 'test2', 'Second test booking', '1');
INSERT INTO youthbooking.booking (startdatetime, enddatetime, roomid, username, info, statusflg) VALUES ('2022-05-02 11:00:00', '2022-05-02 11:30:00', '2', 'test1', 'Third test booking', '1');
INSERT INTO youthbooking.booking (startdatetime, enddatetime, roomid, username, info, statusflg) VALUES ('2022-05-02 12:00:00', '2022-05-02 12:30:00', '2', 'test1', 'Forth test booking', '1');



UPDATE customer SET firstname = 'test1 first name', lastname = 'test1 last name', email = 'test1@tester.com' WHERE (`username` = 'test1');




